unit uReadDocForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, System.Variants, System.IOUtils,
  Class_LotusNotes, Util_LnAPI;

type
  TReadDocDlg = class(TForm)
    Fields: TStringGrid;
    Label1: TLabel;
    btnOk: TButton;
    BtReply: TButton;
    btnCancel: TButton;
    LSign: TLabel;
    BtReply2: TButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BtReplyClick(Sender: TObject);
    procedure BtReply2Click(Sender: TObject);
  private
    Doc: TNotesDocument;
    fMail: boolean;
  public
    procedure OpenDoc(aDoc: TNotesDocument; aMail: boolean);
  end;

var
  ReadDocDlg: TReadDocDlg;

implementation


uses uSendMailForm;


{$R *.DFM}
type
  TFieldMapRec = record
    fieldType: word;
    fieldName: string;
  end;
const
  FieldNameMap: array[0..31] of TFieldMapRec = (
   (fieldType:TYPE_ERROR; fieldName:               'ERROR'),
   (fieldType:TYPE_UNAVAILABLE; fieldName:         'UNAVAILABLE'),
   (fieldType:TYPE_TEXT; fieldName:                'TEXT'),
   (fieldType:TYPE_TEXT_LIST; fieldName:           'TEXT_LIST'),
   (fieldType:TYPE_NUMBER; fieldName:              'NUMBER'),
   (fieldType:TYPE_NUMBER_RANGE; fieldName:        'NUMBER_RANGE'),
   (fieldType:TYPE_TIME; fieldName:                'TIME'),
   (fieldType:TYPE_TIME_RANGE; fieldName:          'TIME_RANGE'),
   (fieldType:TYPE_FORMULA; fieldName:             'FORMULA'),
   (fieldType:TYPE_USERID; fieldName:              'USERID'),
   (fieldType:TYPE_INVALID_OR_UNKNOWN; fieldName:  'INVALID_OR_UNKNOWN'),
   (fieldType:TYPE_COMPOSITE; fieldName:           'COMPOSITE'),
   (fieldType:TYPE_COLLATION; fieldName:           'COLLATION'),
   (fieldType:TYPE_OBJECT; fieldName:              'OBJECT'),
   (fieldType:TYPE_NOTEREF_LIST; fieldName:        'NOTEREF_LIST'),
   (fieldType:TYPE_VIEW_FORMAT; fieldName:         'VIEW_FORMAT'),
   (fieldType:TYPE_ICON; fieldName:                'ICON'),
   (fieldType:TYPE_NOTELINK_LIST; fieldName:       'NOTELINK_LIST'),
   (fieldType:TYPE_SIGNATURE; fieldName:           'SIGNATURE'),
   (fieldType:TYPE_SEAL; fieldName:                'SEAL'),
   (fieldType:TYPE_SEALDATA; fieldName:            'SEALDATA'),
   (fieldType:TYPE_SEAL_LIST; fieldName:           'SEAL_LIST'),
   (fieldType:TYPE_HIGHLIGHTS; fieldName:          'HIGHLIGHTS'),
   (fieldType:TYPE_WORKSHEET_DATA; fieldName:      'WORKSHEET_DATA'),
   (fieldType:TYPE_USERDATA; fieldName:            'USERDATA'),
   (fieldType:TYPE_QUERY; fieldName:               'QUERY'),
   (fieldType:TYPE_ACTION; fieldName:              'ACTION'),
   (fieldType:TYPE_ASSISTANT_INFO; fieldName:      'ASSISTANT_INFO'),
   (fieldType:TYPE_VIEWMAP_DATASET; fieldName:     'VIEWMAP_DATASET'),
   (fieldType:TYPE_VIEWMAP_LAYOUT; fieldName:      'VIEWMAP_LAYOUT'),
   (fieldType:TYPE_LSOBJECT; fieldName:            'LSOBJECT'),
   (fieldType:TYPE_HTML; fieldName:                'HTML')
  );

procedure TReadDocDlg.OpenDoc(aDoc: TNotesDocument; aMail: boolean);
var
 i, j: integer;
 fname, s, s1: string;
 v: variant;
 //
 n: Integer;
 l_sName, l_sFilePath, l_sRunDir: String;
 l_cDirectorySeparatorChar: Char;
begin
 Doc := aDoc;
 fMail := aMail;
 BtReply.Enabled := fMail;
 BtReply2.Enabled := fMail;
 //
 l_cDirectorySeparatorChar := TPath.DirectorySeparatorChar;
 l_sRunDir := TDirectory.GetCurrentDirectory;
 if l_sRunDir[Length(l_sRunDir)] <> l_cDirectorySeparatorChar  then
   l_sRunDir := l_sRunDir + l_cDirectorySeparatorChar;
 //
 n := Doc.AttachmentCount;
 for i := 0 to n-1 do begin
   l_sName := Doc.Attachment[i];
   l_sFilePath := l_sRunDir + l_sName;
   Doc.SaveAttachment(i,l_sFilePath);
 end;
 //
 with Fields do begin
   RowCount := 2;
   Cells[0,0] := 'Name';
   Cells[1,0] := 'Delphi Type';
   Cells[2,0] := 'Notes Type';
   Cells[3,0] := 'Value';
   Cells[0,1] := '';
   Cells[1,1] := '';
   Cells[2,1] := '';
   Cells[3,1] := '';
   if Doc.FieldCount > 0 then begin
     RowCount := Doc.FieldCount+1;
     for i := 0 to Doc.FieldCount-1 do begin
       fname := Doc.FieldName[i];
       Cells[0,i+1] := fname;          // <- Feldname
       try
         v := Doc[fname].Value;
         s := '';
         if (VarType(v) and varArray) <> 0 then
           s := 'Array of ';
         case VarType (v) and varTypeMask of
           varEmpty:    s := s + 'Empty';
           varNull:     s := s + 'Null';
           varSmallint: s := s + 'Smallint';
           varInteger:  s := s + 'Integer';
           varSingle:   s := s + 'Single';
           varDouble:   s := s + 'Double';
           varCurrency: s := s + 'Currency';
           varDate:     s := s + 'TDateTime';
           varOleStr:   s := s + 'OleStr';
           varDispatch: s := s + 'Dispatch';
           varError:    s := s + 'Error';
           varBoolean:  s := s + 'Boolean';
           varVariant:  s := s + 'Variant';
           varUnknown:  s := s + 'Unknown';
           varByte:     s := s + 'Byte';
           varString:   s := s + 'String';
         end;
         // Ermitteln des Wertes
         s1 := '';
         if (not VarIsNull(v)) and (not VarIsEmpty(v)) then begin
           if (VarType(v) and varArray) = 0 then s1 := VarAsType(v, varString)
           else begin
             s1 := '';
             for j := VarArrayLowBound(v,1) to VarArrayHighBound(v,1) do begin
               if (VarType(v) and varTypeMask) = varDate then
                 s1 := s1 + dateTimeToStr (VarToDateTime (v[j])) + '; '
               else
                 s1 := s1 + VarAsType (v[j],varString) + '; ';
             end;
           end;
         end;
       except
         on E: Exception do s1 := 'ERROR: ' + E.Message;
       end;
       Cells[1,i+1] := s;       // <- Delphi type
       for j := System.Low(FieldNameMap) to System.High(FieldNameMap) do begin
         if FieldNameMap[j].fieldType = Doc[fname].itemType then begin
           Cells[2,i+1] := FieldNameMap[j].fieldName;    // <- Notes type
         end;
       end;
       Cells[3,i+1] := s1;     // <- Wert
     end;
   end;
 end;
 if Doc.GetSignature (s, s1, nil) then
   LSign.Caption := 'Signed by ' + s + ' according to ' + s1;
end;

procedure TReadDocDlg.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 Action := caHide;
end;

procedure TReadDocDlg.BtReplyClick(Sender: TObject);
begin
 SendMailDlg.StartReply (Doc, True);
 SendMailDlg.ShowModal;
end;

procedure TReadDocDlg.BtReply2Click(Sender: TObject);
begin
 SendMailDlg.StartReply (Doc, False);
 SendMailDlg.ShowModal;
end;

end.
