unit uViewsFoldersForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, System.UITypes, Class_LotusNotes;

type
  TViewsFoldersForm = class(TForm)
    List: TListBox;
    btnClose: TButton;
    btnListDocs: TButton;
    btnCreateFolder: TButton;
    btnRenameFolder: TButton;
    btnDeleteFolder: TButton;
    btnClearDocs: TButton;
    btnTrim100Docs: TButton;
    procedure FormHide(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnListDocsClick(Sender: TObject);
    procedure btnCreateFolderClick(Sender: TObject);
    procedure btnRenameFolderClick(Sender: TObject);
    procedure btnDeleteFolderClick(Sender: TObject);
    procedure btnClearDocsClick(Sender: TObject);
    procedure btnTrim100DocsClick(Sender: TObject);
  private
    m_oDatabase: TNotesDatabase;
    procedure AddView (aView: TNotesView);
    procedure Clear;
  public
    procedure SetInfo (aDB: TNotesDatabase);
  end;

var
  ViewsFoldersForm: TViewsFoldersForm;

implementation

uses uMainForm;


{$R *.DFM}

procedure TViewsFoldersForm.Clear;
begin
 List.Clear;
end;

procedure TViewsFoldersForm.SetInfo(aDB: TNotesDatabase);
var
 View: TNotesView;
 i: integer;
begin
 m_oDatabase := aDB;
 Clear;
 for i := 0 to m_oDatabase.ViewCount-1 do begin
   View := m_oDatabase.ViewByIndex[i];  //don't need to free
   AddView(View);
 end;
end;

procedure TViewsFoldersForm.FormHide(Sender: TObject);
begin
 Clear;
end;

procedure TViewsFoldersForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 Action := caHide;
end;

procedure TViewsFoldersForm.AddView(aView: TNotesView);
var
 s: string;
begin
 s := aView.Name;
 if aView is TNotesFolder then begin    //or View.IsFolder
   s := s + ' (';
   if TNotesFolder(aView).IsShared then
     s := s + 'shared folder)'
   else
     s := s + 'private folder)';
 end;
 List.Items.AddObject(s,aView);
end;

procedure TViewsFoldersForm.btnListDocsClick(Sender: TObject);
begin
 if List.itemIndex = -1 then
   exit;
 EnumDocs(List.Items.Objects[List.itemIndex] as TNotesView);
end;

procedure TViewsFoldersForm.btnCreateFolderClick(Sender: TObject);
var
  s: string;
  fShared: boolean;
begin
  s := InputBox('Create folder','Enter a folder name','');
  if s = '' then exit;
  case MessageDlg('Create a shared folder ' + s + '?',mtConfirmation,[mbYes,mbNo,mbCancel],0) of
    mrYes: fShared := True;
    mrNo:  fShared := False;
    else exit;
  end;
  TNotesFolder.CreateNew(m_oDatabase, s, fShared, nil);
  SetInfo(m_oDatabase);
end;

procedure TViewsFoldersForm.btnRenameFolderClick(Sender: TObject);
var
  s: string;
  f: TNotesView;
begin
  s := InputBox('Create folder','Enter a folder name','');
  if s = '' then exit;
  with List do begin
    if itemIndex = -1 then exit;
    f := Items.Objects[itemIndex] as TNotesView;
    if not f.IsFolder then exit;
    TNotesFolder(f).Name := s;
  end;
  SetInfo(m_oDatabase);
end;

procedure TViewsFoldersForm.btnDeleteFolderClick(Sender: TObject);
var
 f: TNotesView;
begin
 if List.itemIndex = -1 then
   exit;
 f := List.Items.Objects[List.itemIndex] as TNotesView;
 if not f.IsFolder then
   exit;
 TNotesFolder(f).Delete;
 List.Items.Delete(List.itemIndex);
 SetInfo(m_oDatabase);
end;

procedure TViewsFoldersForm.btnClearDocsClick(Sender: TObject);
var
 l_oNotesView: TNotesView;
begin
 if List.itemIndex = -1 then
   exit;
 l_oNotesView := List.Items.Objects[List.itemIndex] as TNotesView;
 l_oNotesView.Clear;
end;

procedure TViewsFoldersForm.btnTrim100DocsClick(Sender: TObject);
var
 i, j, l_iAttachmentCount, l_iCount:   Integer;
 l_sName: String;
 l_oNotesView:     TNotesView;
 l_oNotesFolder:   TNotesFolder;
 l_oNotesDocument: TNotesDocument;
begin
 if List.itemIndex = -1 then
   exit;
 l_oNotesView := List.Items.Objects[List.itemIndex] as TNotesView;
 l_iCount := l_oNotesView.Count;
 for i := l_iCount-1 downto 100 do begin
   l_oNotesDocument := l_oNotesView.Document[i];
   //
   l_iAttachmentCount := l_oNotesDocument.AttachmentCount;
   for j := l_iAttachmentCount-1 downto 0 do begin
     l_oNotesDocument.DeleteAttachment(j);     // delete an attachment
   end;
   //
   m_oDatabase.Delete(l_oNotesDocument.DocID);
 end;
end;

end.

